// combstr.c

#include "combstr.h"

#if HOSTOS==MSDOS
#include <atlbase.h>
#else
#include "SysString.h"
#endif

#ifdef USE_ALLOC_TRACKER
#include "AllocTracker.h"
#endif

#ifndef NEW
#define NEW new
#endif

size_t bstrlen(BSTR str)
{
#if HOSTOS==MSDOS
	return wcslen(str);
#else
	return strlen(str);
#endif
}

int bstrcmp(BSTR str1, BSTR str2)
{
#if HOSTOS==MSDOS
	return wcscmp(str1, str2);
#else
	return strcmp(str1, str2);
#endif
}

int bstrcmp(const char *str1, BSTR str2)
{
#if HOSTOS==MSDOS
	CComBSTR bstr = str1;
	return wcscmp(bstr, str2);
#else
	return strcmp(str1, str2);
#endif
}

int bstrcmp(BSTR str1, const char *str2)
{
#if HOSTOS==MSDOS
	CComBSTR bstr = str2;
	return wcscmp(str1, bstr);
#else
	return strcmp(str1, str2);
#endif
}

char* bstrcpy(char* str1, BSTR str2)
{
	int len = bstrlen(str2);
#if HOSTOS==MSDOS
	wcstombs(str1, str2, len);	
#else
	strncpy(str1, str2, len);
#endif
	str1[len] = '\0';
	return str1;
}

char* bstrncpy(char* str1, BSTR str2, long len)
{
#if HOSTOS==MSDOS
	wcstombs(str1, str2, len);	
#else
	strncpy(str1, str2, len);
#endif
	return str1;
}

char* bstralloccpy(BSTR str)
{
	if (!str)	return NULL;
	int len = bstrlen(str);
	char* rstr = NEW char[len+1];
	bstrncpy(rstr, str, len);
	rstr[len] = '\0';
	return rstr;
}

unsigned char* bstrtomb(unsigned char* dest, BSTR src, unsigned int size)
{
#if HOSTOS==MSDOS
	for (int i=0;i<size;i++)
		wctomb((char *)&dest[i], src[i]);
#else
	memcpy(dest, src, size);
#endif
	return dest;
}

bool bstrtomb(std::string& dest, BSTR src)
{
	if (!src) return false;
	
#if HOSTOS==MSDOS
	char temp[MB_LEN_MAX+1];
	mbstate_t state;
	memset(&state, 0, sizeof(state));
	
	while (*src)
	{
		int bytes = wcrtomb(temp, *src++, &state);
		if (bytes == -1)
			return false;	// ran into invalid unicode char
			
		temp[bytes] = '\0';
		dest += temp;
	}
#else
	dest = src;
#endif

	return true;
}

BSTR mbtobstr(BSTR dest, unsigned char *src, unsigned int size)
{
#if HOSTOS==MSDOS
	for (int i=0;i<size;i++)
		mbtowc(&dest[i], (const char *)&src[i], MB_CUR_MAX);
#else
	memcpy(dest, src, size);
#endif	
	return dest;
}

/*
 * Note: len is the number of characters to be copied from src. A null character 
 * is placed afterwards, allocating a total of len plus one characters. This
 * is consistent with how SysAllocStringLen works.
 */
void mbtobstrwithallocate(BSTR *dest, const unsigned char *src, unsigned int len)
{
#if HOSTOS==MSDOS
	wchar_t *wideBuffer = NEW wchar_t[len];
	for(int i=0;i<len;i++)
		mbtowc(&wideBuffer[i], (const char *)&src[i], MB_CUR_MAX);
	*dest = ::SysAllocStringLen(wideBuffer, len);
	delete[] wideBuffer;
#else
	BSTR wideBuffer = SysAllocStringLen(NULL, len);
	memcpy(wideBuffer, src, len + 1);
	*dest = wideBuffer;
#endif	
}

//--------------------------------------------------------------------------
/*
	FormatHexLen

	Format raw bytes as hex into formatBuf.

	bytes		start of buffer of bytes to be formatted
	len			number of bytes in the buffer
	formatBuf	output C string
	bufferLength    size of formatBuf in bytes
	lang		programming language
*/
//--------------------------------------------------------------------------
/*
char* FormatHexLen(const char* bytes, long len, char* formatBuf, long bufferLength, ELanguage lang, bool prependIdentifier)
{
	static const char	hexDigits[]="0123456789ABCDEF";
#if MW_HOST_ENDIAN == MW_LITTLE_ENDIAN
	char* uBytes = (char *) (bytes + len - 1);
#else
	char* uBytes = (char *)bytes;
#endif
	long i = 0;	// index into formatBuf
	
	if (prependIdentifier)
	{
		if (lang == langPascal)
		{
			// prepend '$'
			formatBuf[i++] = '$';
		}
		else
		{
			// prepend '0x'
			formatBuf[i++] = '0';
			formatBuf[i++] = 'x';
		}
	}

	for (long j=0; j<len && i < bufferLength - 1; j++)
	{
#if MW_HOST_ENDIAN == MW_LITTLE_ENDIAN
		unsigned char ch = *uBytes--;
#else
		unsigned char ch = *uBytes++;
#endif
		formatBuf[i++] = hexDigits[ch >> 4];
		formatBuf[i++] = hexDigits[ch & 0xF];
	}
	
	formatBuf[i] = 0;	// terminate the string	
	return formatBuf;
}

char* FormatHex(const char* bytes, long len, char* formatBuf, ELanguage lang, bool prependIdentifier) {
    long	bufferLength = 256;	// assuming here
    return FormatHexLen(bytes, len, formatBuf, bufferLength, lang, prependIdentifier);
}
*/
void ScanFromHex(const char* memoryString, char* memBuffer)
{
	long inLen = strlen(memoryString);
	memset(memBuffer, 0, inLen / 2);
	char* outBuf = memBuffer;
	for (long index = 0; index < inLen;)
	{
		char inChar = memoryString[index];
		char outVal = 0;
		if (inChar >= '0' && inChar <= '9')
			outVal = inChar - '0';
		else
		if (inChar >= 'A' && inChar <= 'F')
			outVal = inChar - 55;
		index++;
		
		*outBuf = *outBuf | outVal << 4;
		
		inChar = memoryString[index];
		outVal = 0;
		if (inChar >= '0' && inChar <= '9')
			outVal = inChar - '0';
		else
		if (inChar >= 'A' && inChar <= 'F')
			outVal = inChar - 55;
		index++;
		 
		*outBuf = *outBuf | outVal;
		outBuf++;

	}
}
